package fractals;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;

import javax.swing.JFrame;

/**
 * Fractal Generator Program - Draws a tree fractal and a modified Sierpinski Triangle
 * @author Sergio Castanon-Diaz & Ryan Furukawa
 * Math 2270 - Linear Algebra 
 *
 */
public class FractalGenerator extends JFrame {

	public static void main (String[] args)
	{
		new FractalGenerator().setVisible(true);
	}
	
	/**
	 * Constructor for the generator
	 */
	public FractalGenerator()
	{
		super("Fractal Generator");
        setBounds(100, 100, 800, 600);
        setResizable(false);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
	}
	
	/**
	 * Recursively draws a fractal using squares (final looks like a modified Sierpinski triangle)
	 * @param g - graphics object to draw with
	 * @param x - x coordinate
	 * @param y - y coordinate
	 * @param h - the height of the rectangle
	 * @param w - width of the rectangle
	 * @param iter - number of iterations left to do
	 */
	private void drawSquareTriangle(Graphics2D g, int x, int y, int h, int w, int iter)
	{
		if (iter == 0)
			return;
		

		Rectangle rect = new Rectangle(x, y, w, h);
		
		// only show the final squares
		if (iter == 1)
		{
			g.setColor(Color.WHITE);
		}
		
		AffineTransform transform = new AffineTransform();
		transform.translate(0, -h);
		
		// Get the current transform
		AffineTransform saveAT = g.getTransform();
		// Perform transformation
		g.transform(transform);
		
		// Render
		g.draw(rect);
		g.fill(rect);
		
		// Restore original transform
		g.setTransform(saveAT);
		g.setColor(Color.BLACK);
		
		// F(x, y) = (1/2x, 1/2y)
        drawSquareTriangle(g, x, y, h/2, w/2, iter - 1);
        
        // F(x, y) = (1/2x + (w/2), 1/2y)
        drawSquareTriangle(g, x+(w/2), y, h/2, w/2, iter - 1);

        // F(x, y) = (1/2x, 1/2y - h/2)
        drawSquareTriangle(g, x, y-(h/2), h/2, w/2, iter-1);
	}
	
	/**
	 * Recursive method for drawing a tree fractal
	 * 
	 * @param g - graphics object
	 * @param x - x coordinate
	 * @param y - y coordinate
	 * @param length - length of the line for the "tree branch"
	 * @param angle_delta - angle at which to rotate the new branches
	 * @param iter - number of iterations left
	 */
	private void drawTree(Graphics2D g, int x, int y, int length, int angle, int angle_delta, int iter)
	{
		if (iter == 0)
			return;
		
		int x2 = x + ((int) (Math.cos(Math.toRadians(angle)) * length));
		int y2 = y + ((int) (Math.sin(Math.toRadians(angle)) * length));
		
		g.drawLine(x, y, x2, y2);
			
		
		drawTree(g, x2, y2, (int)(length/1.5), angle + angle_delta, angle_delta, iter - 1);
		
		drawTree(g, x2, y2, (int)(length/1.5), angle - angle_delta, angle_delta, iter - 1);
	}
	
	/**
	 * Starts the fractal drawing process (only two methods, one is commented out)
	 */
	public void paint(Graphics g)
	{
		/* change this variable for the number of iterations (we found 9 to be sufficient for 
		 * the square triangle fractal and about 16 or so for the tree fractal) */
		int iterations = 15;
		Graphics2D g2 = (Graphics2D) g;
        g2.setColor(Color.BLACK);
        /* uncomment the fractal that you wish to be drawn */
        
        //drawSquareTriangle(g2, 200, 500, 400, 400, iterations);
        drawTree(g2, 400, 600, 180, -90, 120, iterations);
	}
}
